#ifndef COMPRESSER_RLE_H
#define COMPRESSER_RLE_H

#include "rgb_value.h"
#include <vector>
#include <cassert>
#include <limits>
#include <type_traits>
#include <algorithm>
#include <cstdint>

template <class Dest, class Src>
   std::vector<Dest> compresser_rle(const std::vector<Src> &v) {
      using namespace std;
      //
      // Rgles:
      // - Src peut tre converti de manire statique en Dest
      // - une diffrence d'itrateurs peut tre convertie de manire statique en Dest
      //
      static_assert(is_convertible<Src,Dest>::value);
      static_assert(is_convertible_v<
         typename iterator_traits<vector<Src>::iterator>::difference_type, Dest
      >);
      //
      // Le code
      //
      vector<Dest> v_rle;
      typename vector<Dest>::size_type pos = 0, cur = 1;
      Src tgt = v[0];
      //
      //
      //
      for (; cur != v.size(); ++cur) {
         auto n = cur - pos;
         if (v[cur] != tgt || n == numeric_limits<Dest>::max()) {
            v_rle.push_back(static_cast<Dest>(n));
            v_rle.push_back(static_cast<Dest>(tgt));
            tgt = v[cur];
            pos = cur;
         }
      }
      auto n = cur - pos;
      if (n) {
         v_rle.push_back(static_cast<Dest>(n));
         v_rle.push_back(static_cast<Dest>(tgt));
      }
      return v_rle;
   }

#endif
