#ifndef RGB_VALUE_H
#define RGB_VALUE_H

#include <cstdint>

class rgb_value {
public:
   using byte = std::uint8_t;
private:
   enum { ROUGE, VERT, BLEU, ALPHA, NBYTES };
   byte values[NBYTES]{}; // R G B (A)
public:
   rgb_value() = default;
   rgb_value(byte r, byte g, byte b) noexcept {
      red(r);
      green(g);
      blue(b);
      alpha({});
   }
   void red(byte val) noexcept {
      values[ROUGE] = val;
   }
   byte red() const noexcept {
      return values[ROUGE];
   }
   void green(byte val) noexcept {
      values[VERT] = val;
   }
   byte green() const noexcept {
      return values[VERT];
   }
   void blue(byte val) noexcept {
      values[BLEU] = val;
   }
   byte blue() const noexcept {
      return values[BLEU];
   }
   void alpha(byte val) noexcept {
      values[ALPHA] = val;
   }
   byte alpha() const noexcept {
      return values[ALPHA];
   }
   operator std::uint32_t() const noexcept {
      return *reinterpret_cast<const std::uint32_t*>(values);
   }
   bool operator==(const rgb_value &autre) const noexcept {
      return static_cast<std::uint32_t>(*this) == static_cast<std::uint32_t>(autre);
   }
   bool operator!=(const rgb_value &autre) const noexcept {
      return !(*this == autre);
   }
};

#include <vector>
std::vector<rgb_value> to_rgb_values(const std::vector<std::uint8_t> &);

#endif
